/*******************************************
 ***  CRT display simulator for GORODKI  ***
 *******************************************/

#pragma comment (lib, "gdi32.lib")
#pragma comment (lib, "user32.lib")

#include <windows.h>   
#include "vsm.hpp"   

#define SIZE_X  246*2
#define SIZE_Y  330*2


class DISPLAY : public IACTIVEMODEL, public IDSIMMODEL
{
public:
	int isdigital(CHAR *pinname);
	void setup(IINSTANCE *inst, IDSIMCKT *dsim);
	void runctrl(RUNMODES mode);
	void actuate(REALTIME time, ACTIVESTATE newstate);
	BOOL indicate(REALTIME time, ACTIVEDATA *data);
	void simulate(ABSTIME time, DSIMMODES mode);
	void callback(ABSTIME time, EVENTID eventid);
	void initialize(ICOMPONENT *cpt);
	ISPICEMODEL *getspicemodel(CHAR *device);
	IDSIMMODEL *getdsimmodel(CHAR *device);
	void plot(ACTIVESTATE state);
	void animate(INT element, ACTIVEDATA *newstate);
	BOOL actuate(WORD key, INT x, INT y, DWORD flags);

private:
	IINSTANCE *instance;
	IDSIMCKT *ckt;
	ICOMPONENT *component;
	IDSIMPIN *en_video;
	IDSIMPIN *en_sync_h;
	IDSIMPIN *en_sync_v;
	IDSIMPIN *en_pclk;
	IDSIMPIN *en_bright;
};

HANDLE hProcessHeap;
HDC hPopupDC, hDblBufDC;
HBITMAP hBmp;
PCHAR pDrawBuf, pDblBuf;
HWND hPopupWnd = 0;
BITMAPINFO bmi;

IUSERPOPUP  *popup;
IMSGHLR * popupCallback;




extern "C" IDSIMMODEL __declspec(dllexport)*createdsimmodel(CHAR *device, ILICENCESERVER *ils) {
//	ils->authorize(0x88888888, 0x69);
	ils->authorize(0);
	return new DISPLAY;
}

extern "C" VOID __declspec(dllexport)deletedsimmodel(IDSIMMODEL*model) {
	HeapFree(hProcessHeap, 0, pDrawBuf);
	HeapFree(hProcessHeap, 0, pDblBuf);

	if (popup != NULL)
	{
		delete popup;
		delete popupCallback;
	}
	delete (DISPLAY*)model;
}

extern "C" IACTIVEMODEL __declspec(dllexport)*createactivemodel(CHAR *device, ILICENCESERVER *ils) {
	ils->authorize(0);
	return new DISPLAY;
}

extern "C" VOID  __declspec(dllexport)deleteactivemodel(IACTIVEMODEL *model) {
	delete (DISPLAY*)model;
}





class WINCB : public IMSGHLR
{
public:
	LRESULT msghlr(HWND, MESSAGE, WPARAM, LPARAM);

};

LRESULT WINCB::msghlr(HWND hwnd, MESSAGE msg, WPARAM wParam, LPARAM lParam)
{
	if (!hPopupWnd)
	{
		hPopupWnd = hwnd;

		hPopupDC = GetDC(hwnd);
		hDblBufDC = CreateCompatibleDC(hPopupDC);
		ReleaseDC(hwnd, hPopupDC);

		bmi.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);

		bmi.bmiHeader.biWidth = SIZE_X;
		bmi.bmiHeader.biHeight = -SIZE_Y;
		bmi.bmiHeader.biPlanes = 1;
		bmi.bmiHeader.biBitCount = 8;
		bmi.bmiHeader.biCompression = BI_RGB;
		bmi.bmiHeader.biSizeImage = 0;
		bmi.bmiHeader.biXPelsPerMeter = 0;
		bmi.bmiHeader.biYPelsPerMeter = 0;
		bmi.bmiHeader.biClrUsed = 0;
		bmi.bmiHeader.biClrImportant = 0;

		bmi.bmiColors[0x80].rgbBlue = 0;
		bmi.bmiColors[0x80].rgbGreen = 0;
		bmi.bmiColors[0x80].rgbRed = 0;
		bmi.bmiColors[0x80].rgbReserved = 0;

		bmi.bmiColors[0x81].rgbBlue = 120;
		bmi.bmiColors[0x81].rgbGreen = 120;
		bmi.bmiColors[0x81].rgbRed = 90;
		bmi.bmiColors[0x81].rgbReserved = 0;

		bmi.bmiColors[0x82].rgbBlue = 240;
		bmi.bmiColors[0x82].rgbGreen = 240;
		bmi.bmiColors[0x82].rgbRed = 180;
		bmi.bmiColors[0x82].rgbReserved = 0;

		hBmp = CreateDIBSection(hDblBufDC, &bmi, DIB_RGB_COLORS, (void**)pDblBuf, NULL, 0);
		SelectObject(hDblBufDC, hBmp);
	}

	return popup->callwindowproc(msg, wParam, lParam);
}




int DISPLAY::isdigital(CHAR *pinname) {
	return 1;
}

void DISPLAY::setup(IINSTANCE *inst, IDSIMCKT *dsim) {
	instance = inst;
	ckt = dsim;

	hProcessHeap = GetProcessHeap();
	pDrawBuf = (PCHAR)HeapAlloc(hProcessHeap, 0, (SIZE_Y + 2) * SIZE_X);
	pDblBuf = (PCHAR)HeapAlloc(hProcessHeap, 0, (SIZE_Y + 2) * SIZE_X);

	CREATEPOPUPSTRUCT cp;
	cp.caption = "CRT SCREEN";
	cp.id = 0;
	cp.type = PWT_USER;
	cp.flags = PWF_VISIBLE;
	cp.width = SIZE_X;
	cp.height = SIZE_Y;

	popup = (IUSERPOPUP*)instance->createpopup(&cp);

	if (popup != NULL)
	{
		popupCallback = new WINCB;
		popup->setmsghlr(popupCallback);
	}

	en_video = instance->getdsimpin("VIDEO", true);
	en_sync_h = instance->getdsimpin("SYNC_H", true);
	en_sync_v = instance->getdsimpin("SYNC_V", true);
	en_pclk = instance->getdsimpin("PCLK", true);
	en_bright = instance->getdsimpin("BRT", true);
}

BOOL DISPLAY::indicate(REALTIME time, ACTIVEDATA *data) {
	hPopupDC = GetDC(hPopupWnd);
	BitBlt(hPopupDC, 0, 0, SIZE_X, SIZE_Y, hDblBufDC, 0, 0, SRCCOPY);
	ReleaseDC(hPopupWnd, hPopupDC);
	return TRUE;
}

void DISPLAY::simulate(ABSTIME time, DSIMMODES mode) {
	UCHAR c;

	static int y = 0, x = 0;
	static BOOL pr_hsync = 1, pr_vsync = 1, pr_clock = 1;

	BOOL vid = ishigh(en_video->istate());
	BOOL hsync = ishigh(en_sync_h->istate());
	BOOL vsync = ishigh(en_sync_v->istate());
	BOOL clock = ishigh(en_pclk->istate());
	BOOL brt = ishigh(en_bright->istate());

	if (vsync)
	{
		pr_vsync = 1;
		if (hsync)
		{
			pr_hsync = 1;
			if (clock)
			{
				pr_clock = 1;
			}
			else
			{
				if (pr_clock) {
					pr_clock = 0;
					if (vid)  c = 0x80;  else
						if (brt) c = 0x82; else c = 0x81;

					*(pDrawBuf + y + x) =
						*(pDrawBuf + y + x + 1) =
						*(pDrawBuf + y + x + SIZE_X) =
						*(pDrawBuf + y + x + SIZE_X + 1) = c;
					x += 2;
					if (x > SIZE_X) x = 0;
				}
			}
		}
		else
		{
			if (pr_hsync)
			{
				pr_hsync = 0;
				x = 0;
				y += SIZE_X * 2;
				if (y > (SIZE_Y*SIZE_X)) y = 0;
			}
		}
	}
	else
	{
		if (pr_vsync)
		{
			pr_vsync = 0;
			y = 0; x = 0;
			SetBitmapBits(hBmp, SIZE_X*SIZE_Y, pDrawBuf);
		}
	}
}

void DISPLAY::runctrl(RUNMODES mode) {

}

void DISPLAY::actuate(REALTIME time, ACTIVESTATE newstate) {

}

void DISPLAY::callback(ABSTIME time, EVENTID eventid) {

}

void DISPLAY::initialize(ICOMPONENT *cpt) {

}

ISPICEMODEL *DISPLAY::getspicemodel(CHAR *) {
	return NULL;
}

IDSIMMODEL  *DISPLAY::getdsimmodel(CHAR *) {
	return this;
}

void DISPLAY::plot(ACTIVESTATE state) {

}

void DISPLAY::animate(INT element, ACTIVEDATA *data) {

}

BOOL DISPLAY::actuate(WORD key, INT x, INT y, DWORD flags) {
	return FALSE;
}
